@@ -20,21 +20,21 @@
 
 #include <time.h>
 
+#include <QDateTime>
 #include <QStringList>
 
 #include "connector.h"
+#include "glasslimits.h"
 #include "logging.h"
 
-Connector::Connector(QObject *parent)
+Connector::Connector(const QString &mimetype,QObject *parent)
   : QObject(parent)
 {
   conn_server_username="source";
   conn_server_password="";
-  conn_server_mountpoint="";
-  conn_content_type="";
-  //  conn_audio_channels=2;
-  //  conn_audio_samplerate=44100;
-  conn_audio_bitrates.push_back(128);
+  conn_audio_channels=0;
+  conn_audio_samplerate=0;
+  conn_audio_bitrates.push_back(0);
   conn_stream_name="no name";
   conn_stream_description="unknown";
   conn_stream_url="";
@@ -48,43 +48,22 @@ Connector::Connector(QObject *parent)
   conn_host_hostname="";
   conn_host_port=0;
   conn_connected=false;
-  /*
-  conn_watchdog_active=false;
-  conn_script_up_process=NULL;
-  conn_script_down_process=NULL;
+  conn_connected_changed=true;
+  conn_dropouts=0;
+  conn_dropouts_changed=true;
+  conn_content_type=mimetype;
+  conn_start_metadata=false;
 
-  conn_data_timer=new QTimer(this);
-  connect(conn_data_timer,SIGNAL(timeout()),this,SLOT(dataTimeoutData()));
-  conn_data_timer->start(RINGBUFFER_SERVICE_INTERVAL);
-
-  conn_watchdog_timer=new QTimer(this);
-  conn_watchdog_timer->setSingleShot(true);
-  connect(conn_watchdog_timer,SIGNAL(timeout()),
-	  this,SLOT(watchdogTimeoutData()));
-
-  conn_stop_timer=new QTimer(this);
-  conn_stop_timer->setSingleShot(true);
-  connect(conn_stop_timer,SIGNAL(timeout()),this,SLOT(stopTimeoutData()));
-
-  conn_script_down_garbage_timer=new QTimer(this);
-  conn_script_down_garbage_timer->setSingleShot(true);
-  connect(conn_script_down_garbage_timer,SIGNAL(timeout()),
-	  this,SLOT(scriptDownCollectGarbageData()));
-
-  conn_script_up_garbage_timer=new QTimer(this);
-  conn_script_up_garbage_timer->setSingleShot(true);
-  connect(conn_script_up_garbage_timer,SIGNAL(timeout()),
-	  this,SLOT(scriptUpCollectGarbageData()));
-  */
+  for(unsigned i=0;i<Codec::TypeLast;i++) {
+    if(Codec::acceptsContentType((Codec::Type)i,mimetype)) {
+      setCodecType((Codec::Type)i);
+    }
+  }
 }
 
 
 Connector::~Connector()
 {
-  /*
-  delete conn_stop_timer;
-  delete conn_data_timer;
-  */
 }
 
 
@@ -112,27 +91,55 @@ void Connector::setServerPassword(const QString &str)
 }
 
 
+QUrl Connector::serverUrl() const
+{
+  return conn_server_url;
+}
+
+
+void Connector::setServerUrl(const QUrl &url)
+{
+  conn_server_url=url;
+  uint16_t port=conn_server_url.port();
+  if(port==65535) {
+    conn_server_url.setPort(DEFAULT_SERVER_PORT);
+  }
+  if(conn_server_url.path().isEmpty()) {
+    conn_server_url.setPath("/");
+  }
+}
+
+
+QUrl Connector::publicUrl() const
+{
+  if(conn_public_url.toString().isEmpty()) {
+    return conn_server_url;
+  }
+  return conn_public_url;
+}
+
+
+void Connector::setPublicUrl(const QUrl &url)
+{
+  conn_public_url=url;
+}
+
+
 QString Connector::serverMountpoint() const
 {
-  return conn_server_mountpoint;
+  return conn_server_url.path();
 }
 
 
-void Connector::setServerMountpoint(const QString &str)
+unsigned Connector::audioChannels() const
 {
-  conn_server_mountpoint=str;
+  return conn_audio_channels;
 }
 
 
-QString Connector::contentType() const
+unsigned Connector::audioSamplerate() const
 {
-  return conn_content_type;
-}
-
-
-void Connector::setContentType(const QString &str)
-{
-  conn_content_type=str;
+  return conn_audio_samplerate;
 }
 
 
@@ -164,90 +171,6 @@ void Connector::setAudioBitrates(std::vector<unsigned> *rates)
 }
 
 
-QString Connector::streamName() const
-{
-  return conn_stream_name;
-}
-
-
-void Connector::setStreamName(const QString &str)
-{
-  conn_stream_name=str;
-}
-
-
-QString Connector::streamDescription() const
-{
-  return conn_stream_description;
-}
-
-
-void Connector::setStreamDescription(const QString &str)
-{
-  conn_stream_description=str;
-}
-
-
-QString Connector::streamUrl() const
-{
-  return conn_stream_url;
-}
-
-
-void Connector::setStreamUrl(const QString &str)
-{
-  conn_stream_url=str;
-}
-
-
-QString Connector::streamIrc() const
-{
-  return conn_stream_irc;
-}
-
-
-void Connector::setStreamIrc(const QString &str)
-{
-  conn_stream_irc=str;
-}
-
-
-QString Connector::streamIcq() const
-{
-  return conn_stream_icq;
-}
-
-
-void Connector::setStreamIcq(const QString &str)
-{
-  conn_stream_icq=str;
-}
-
-
-QString Connector::streamAim() const
-{
-  return conn_stream_aim;
-}
-
-
-void Connector::setStreamAim(const QString &str)
-{
-  conn_stream_aim=str;
-}
-
-
-QString Connector::streamGenre() const
-{
-  return conn_stream_genre;
-}
-
-
-void Connector::setStreamGenre(const QString &str)
-{
-  conn_stream_genre=str;
-}
-
-
 bool Connector::streamMetadataEnabled() const
 {
   return conn_stream_metadata_enabled;
@@ -260,68 +183,46 @@ void Connector::setStreamMetadataEnabled(bool state)
 }
 
 
-QString Connector::streamMetadata() const
+void Connector::startMetadata()
 {
-  return conn_stream_metadata;
+  emit metadataReceived(0,&conn_metadata);
+  conn_metadata.processed();
+  conn_start_metadata=true;
 }
 
 
-void Connector::setStreamMetadata(const QString &str)
+MetaEvent *Connector::metadataEvent()
 {
-  if(str!=conn_stream_metadata) {
-    conn_stream_metadata=str;
-    emit streamMetadataChanged(str);
-  }
+  return &conn_metadata;
 }
 
 
-bool Connector::streamPublic() const
+Codec::Type Connector::codecType() const
 {
-  return conn_stream_public;
+  return conn_codec_type;
 }
 
 
-void Connector::setStreamPublic(bool state)
+QString Connector::contentType() const
 {
-  conn_stream_public=state;
+  return conn_content_type;
 }
 
 
-QString Connector::extension() const
+bool Connector::isConnected() const
 {
-  return conn_extension;
+  return conn_connected;
 }
 
 
-void Connector::setExtension(const QString &str)
+void Connector::connectToServer()
 {
-  conn_extension=str;
-}
-
-
-QString Connector::formatIdentifier() const
-{
-  return conn_format_identifier;
-}
-
-
-void Connector::setFormatIdentifier(const QString &str)
-{
-  conn_format_identifier=str;
-}
-
-
-void Connector::connectToServer(const QString &hostname,uint16_t port)
-{
-  conn_host_hostname=hostname;
-  conn_host_port=port;
-  connectToHostConnector(hostname,port);
+  connectToHostConnector();
 }
 
 
 void Connector::stop()
 {
-  startStopping();
   setConnected(false);
 }
 
@@ -350,15 +251,56 @@ void Connector::setScriptDown(const QString &cmd)
 }
 
 
+void Connector::getStats(QStringList *hdrs,QStringList *values,bool is_first)
+{
+  if(conn_connected_changed) {
+    hdrs->push_back("Connector|Connected");
+    if(conn_connected) {
+      values->push_back("Yes");
+    }
+    else {
+      values->push_back("No");
+    }
+    conn_connected_changed=false;
+  }
+
+  if(is_first) {
+    hdrs->push_back("Connector|Url");
+    values->push_back(conn_public_url.toString());
+
+    if(conn_public_url.path()!=conn_server_url.path()) {
+      hdrs->push_back("Connector|InternalUrl");
+      values->push_back(conn_server_url.toString());
+    }
+  }
+
+  if(conn_dropouts_changed) {
+    hdrs->push_back("Connector|Dropouts");
+    values->push_back(QString().sprintf("%u",conn_dropouts));
+    conn_dropouts_changed=false;
+  }
+
+  loadStats(hdrs,values,is_first);
+}
+
+
 QString Connector::serverTypeText(Connector::ServerType type)
 {
   QString ret=tr("Unknown");
 
   switch(type) {
+  case Connector::HlsServer:
+    ret=tr("HTTP Live Stream (HLS)");
+    break;
+
   case Connector::XCastServer:
     ret=tr("IceCast or Shoutcast");
     break;
 
+  case Connector::FileServer:
+    ret=tr("File");
+    break;
+
   case Connector::LastServer:
     break;
   }
@@ -372,10 +314,17 @@ QString Connector::optionKeyword(Connector::ServerType type)
   QString ret;
 
   switch(type) {
+  case Connector::HlsServer:
+    ret="hls";
+    break;
+
   case Connector::XCastServer:
     ret="xcast";
     break;
 
+  case Connector::FileServer:
+    ret="file";
+    break;
 
   case Connector::LastServer:
     break;
@@ -399,6 +348,29 @@ Connector::ServerType Connector::serverType(const QString &key)
 }
 
 
+bool Connector:: acceptsContentType(Connector::ServerType type,
+				    const QString &mimetype)
+{
+  bool ret=false;
+
+  switch(type) {
+  case Connector::XCastServer:
+    ret=(mimetype.toLower()=="audio/mpeg")||
+      (mimetype.toLower()=="audio/aacp");
+    break;
+
+  case Connector::HlsServer:    // We don't list any mimetypes here because
+  case Connector::FileServer:   // ServerId handles it.
+    break;
+
+  case Connector::LastServer:
+    break;
+  }
+
+  return ret;
+}
+
+
 QString Connector::subMountpointName(const QString &mntpt,unsigned bitrate)
 {
   QStringList f0=mntpt.split(".");
@@ -427,125 +399,153 @@ QString Connector::basePart(const QString &fullpath)
 }
 
 
-void Connector::startStopping()
+QDateTime Connector::xmlTimestamp(const QString &str)
 {
-  //  conn_stop_timer->start(0);
+  QDate date;
+  QTime time;
+  QDateTime ret;
+  QString time_str;
+  QString tz_str;
+  int tz_offset=0;
+
+  QStringList f0=str.split("T");
+  if(f0.size()==2) {
+    //
+    // Extract Date
+    //
+    QStringList f1=f0[0].split("-");
+    if(f1.size()==3) {
+      date=QDate(f1[0].toInt(),f1[1].toInt(),f1[2].toInt());
+    }
+
+    //
+    // Extract Time
+    //
+    f1=f0[1].split("+");
+    if(f1.size()==2)  {
+      time_str=f1[0];
+      tz_str="+"+f1[1];
+    }
+    else {
+      f1=f0[1].split("-");
+      if(f1.size()==2)  {
+	time_str=f1[0];
+	tz_str="-"+f1[1];
+      }
+      else {
+	if(f0[1].right(1).toLower()=="z") {  // UTC
+	  time_str=f0[1].left(f0[1].length()-1);
+	  tz_str="+00:00";
+	}
+	else {   // No TZ info, assume local time
+	  time_str=f0[1];
+	  tz_str="+00:00";
+	}
+      }
+    }
+    QStringList f2=time_str.split(":");
+    if(f2.size()==3) {
+      QStringList f3=f2[2].split(".");
+      if(f3.size()==2) {
+	time=QTime(f2[0].toInt(),f2[1].toInt(),f3[0].toInt(),f3[1].toInt());
+      }
+      else {
+	time=QTime(f2[0].toInt(),f2[1].toInt(),f2[2].toInt());
+      }
+      f2=tz_str.right(tz_str.length()-1).split(":");
+      if(f2.size()==2) {
+	tz_offset=60*f2[0].toInt()+f2[1].toInt();
+	if(tz_str.left(1)=="-") {
+	  tz_offset=-tz_offset;
+	}
+      }
+    }
+  }
+  if(date.isValid()&&time.isValid()) {
+    ret=QDateTime(date,time);
+    ret.addSecs(Connector::timezoneOffset()-tz_offset);
+  }
+
+  return ret;
+}
+
+
+QString Connector::timezoneOffsetString()
+{
+  QString ret="Z";
+  time_t t=time(NULL);
+  time_t gmt;
+  time_t lt;
+  
+  gmt=mktime(gmtime(&t));
+  lt=mktime(localtime(&t));
+
+  if(gmt<lt) {
+    ret=QString().sprintf("-%02ld:%02ld",(lt-gmt)/3600,((lt-gmt)%3600)/60);
+  }
+  if(gmt>lt) {
+    ret=QString().sprintf("+%02ld:%02ld",(gmt-lt)/3600,((gmt-lt)%3600)/60);
+  }
+
+  return ret;
+}
+
+
+int Connector::timezoneOffset()
+{
+  time_t gmt;
+  time_t lt;
+  time_t t=time(NULL);
+  
+  gmt=mktime(gmtime(&t));
+  lt=mktime(localtime(&t));
+
+  return (gmt-lt)/60;
+}
+
+
+void Connector::setAudioChannels(unsigned chans)
+{
+  conn_audio_channels=chans;
+}
+
+
+void Connector::setAudioSamplerate(unsigned samprate)
+{
+  conn_audio_samplerate=samprate;
+}
+
+
+void Connector::setCodecType(Codec::Type type)
+{
+  conn_codec_type=type;
 }
 
 
 void Connector::setConnected(bool state)
 {
-  /*
-  QStringList args;
-  QString cmd;
-
-  if(conn_connected!=state) {
-    if(state) {
-      if(conn_script_up_process==NULL) {
-	if(!conn_script_up.isEmpty()) {
-	  args=conn_script_up.split(" ");
-	  cmd=args[0];
-	  args.erase(args.begin());
-	  conn_script_up_process=new QProcess(this);
-	  connect(conn_script_up_process,SIGNAL(error(QProcess::ProcessError)),
-		  this,SLOT(scriptErrorData(QProcess::ProcessError)));
-	  connect(conn_script_up_process,
-		  SIGNAL(finished(int,QProcess::ExitStatus)),
-		  this,SLOT(scriptUpFinishedData(int,QProcess::ExitStatus)));
-	  conn_script_up_process->start(cmd,args);
-	}
-      }
-      else {
-	if(global_log_verbose) {
-	  Log(LOG_WARNING,"curl(1) script-up command overrun, cmd: \""+
-	      args.join(" ")+"\"");
-	}
-	else {
-	  Log(LOG_WARNING,"curl(1) command overrun");
-	}
-      }
-    }
-    else {
-      if(conn_script_down_process==NULL) {
-	if(!conn_script_down.isEmpty()) {
-	  args=conn_script_down.split(" ");
-	  cmd=args[0];
-	  args.erase(args.begin());
-	  conn_script_down_process=new QProcess(this);
-	  connect(conn_script_down_process,SIGNAL(error(QProcess::ProcessError)),
-		  this,SLOT(scriptErrorData(QProcess::ProcessError)));
-	  connect(conn_script_down_process,
-		  SIGNAL(finished(int,QProcess::ExitStatus)),
-		  this,SLOT(scriptDownFinishedData(int,QProcess::ExitStatus)));
-	  conn_script_down_process->start(cmd,args);
-	}
-      }
-      else {
-	if(global_log_verbose) {
-	  Log(LOG_WARNING,"curl(1) script-down command overrun, cmd: \""+
-	      args.join(" ")+"\"");
-	}
-	else {
-	  Log(LOG_WARNING,"curl(1) command overrun");
-	}
-      }
+  if(state!=conn_connected) {
+    if(!state) {
+      conn_dropouts++;
+      conn_dropouts_changed=true;
+      conn_start_metadata=false;
     }
+    conn_connected=state;
+    conn_connected_changed=true;
+    emit connected(conn_connected);
   }
-  if(state&&conn_watchdog_active) {
-    if(conn_server_mountpoint.isEmpty()) {
-      Log(LOG_WARNING,
-	  QString().sprintf("connection to \"%s:%u\" restored",
-			    (const char *)conn_host_hostname.toUtf8(),
-			    0xFFFF&conn_host_port));
-    }
-    else {
-      Log(LOG_WARNING,
-	  QString().sprintf("connection to \"%s:%u/%s\" restored",
-	       (const char *)conn_host_hostname.toUtf8(),0xFFFF&conn_host_port,
-			    (const char *)conn_server_mountpoint.toUtf8()));
-    }
-    conn_watchdog_active=false;
-  }
-  conn_connected=state;
-  emit connected(state);
-  */
 }
 
 
-void Connector::setError(QAbstractSocket::SocketError err)
+void Connector::setMetadataField(uint64_t bytes,MetaEvent::Field field,
+				 const QVariant &value)
 {
-  /*
-  if(!conn_watchdog_active) {
-    if(conn_server_mountpoint.isEmpty()) {
-      Log(LOG_WARNING,
-	  QString().sprintf("connection to \"%s:%u\" lost",
-			    (const char *)conn_host_hostname.toUtf8(),
-			    0xFFFF&conn_host_port));
-    }
-    else {
-      Log(LOG_WARNING,
-	  QString().sprintf("connection to \"%s:%u/%s\" lost",
-			    (const char *)conn_host_hostname.toUtf8(),
-			    0xFFFF&conn_host_port,
-			    (const char *)conn_server_mountpoint.toUtf8()));
-    }
-    conn_watchdog_active=true;
+  conn_metadata.setField(field,value);
+
+  if(conn_start_metadata) {
+    emit metadataReceived(bytes,&conn_metadata);
+    conn_metadata.processed();
   }
-  disconnectFromHostConnector();
-  conn_watchdog_timer->start(5000);
-  */
-}
-
-
-QString Connector::hostHostname() const
-{
-  return conn_host_hostname;
-}
-
-
-uint16_t Connector::hostPort() const
-{
-  return conn_host_port;
 }
 
 
@@ -1220,21 +1220,87 @@ QString Connector::httpStrError(int status_code)
 }
 
 
-QString Connector::timezoneOffset()
+QString Connector::socketErrorText(QAbstractSocket::SocketError err)
 {
-  QString ret="Z";
-  time_t t=time(NULL);
-  time_t gmt;
-  time_t lt;
-  
-  gmt=mktime(gmtime(&t));
-  lt=mktime(localtime(&t));
+  QString ret=tr("Unknown socket error")+QString().sprintf(" [%u]",err);
 
-  if(gmt<lt) {
-    ret=QString().sprintf("-%02ld:%02ld",(lt-gmt)/3600,((lt-gmt)%3600)/60);
+  switch(err) {
+  case QAbstractSocket::ConnectionRefusedError:
+    ret=tr("connection refused");
+    break;
+
+  case QAbstractSocket::RemoteHostClosedError:
+    ret=tr("remote host closed connection");
+    break;
+
+  case QAbstractSocket::HostNotFoundError:
+    ret=tr("host not found");
+    break;
+
+  case QAbstractSocket::SocketAccessError:
+    ret=tr("socket access error");
+    break;
+
+  case QAbstractSocket::SocketTimeoutError:
+    ret=tr("operation timed out");
+    break;
+
+  case QAbstractSocket::DatagramTooLargeError:
+    ret=tr("datagram too large");
+    break;
+
+  case QAbstractSocket::NetworkError:
+    ret=tr("network error");
+    break;
+
+  case QAbstractSocket::AddressInUseError:
+    ret=tr("address in use");
+    break;
+
+  case QAbstractSocket::SocketAddressNotAvailableError:
+    ret=tr("address not available");
+    break;
+
+  case QAbstractSocket::UnsupportedSocketOperationError:
+    ret=tr("unsupported socket operation");
+    break;
+
+  default:
+    break;
   }
-  if(gmt>lt) {
-    ret=QString().sprintf("+%02ld:%02ld",(gmt-lt)/3600,((gmt-lt)%3600)/60);
+
+  return ret;
+}
+
+
+QString Connector::processErrorText(QProcess::ProcessError err)
+{
+  QString ret=tr("Unknown process error")+QString().sprintf("[%u]",err);
+
+  switch(err) {
+  case QProcess::FailedToStart:
+    ret=tr("failed to start");
+    break;
+
+  case QProcess::Crashed:
+    ret=tr("crashed");
+    break;
+
+  case QProcess::Timedout:
+    ret=tr("timed out");
+    break;
+
+  case QProcess::WriteError:
+    ret=tr("write error");
+    break;
+
+  case QProcess::ReadError:
+    ret=tr("read error");
+    break;
+
+  case QProcess::UnknownError:
+    ret=tr("unknown error");
+    break;
   }
 
   return ret;
