@@ -18,6 +18,7 @@
 //   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 //
 
+#include <stdlib.h>
 #include <unistd.h>
 
 #include "codec_null.h"
@@ -57,7 +58,15 @@ bool CodecNull::acceptsFormatIdentifier(const QString &str) const
 }
 
 
-void CodecNull::process(const QByteArray &data)
+void CodecNull::process(const QByteArray &data,bool is_last)
 {
   write(1,data.data(),data.length());
+  if(is_last) {
+    exit(0);
+  }
+}
+
+
+void CodecNull::loadStats(QStringList *hdrs,QStringList *values,bool is_first)
+{
 }
